# ================================================== #
# ============ # Levels of selection # ============= #
# ================================================== #

# ==================== #
# Administration setup #
# ==================== #
rm(list = ls())

# Install/load packages
if(!require(tidyverse)){ install.packages("tidyverse") }; library(tidyverse)
if(!require(viridis)){ install.packages("viridis") }; library(viridis)
if(!require(cowplot)){ install.packages("cowplot") }; library(cowplot)



## if the pool installation and loading of "tidyverse" does not work,
## run the follwoing chunk if the script, which installes and loads the packages indiviudally
## if it did work, skip line 22 - 35 . 

install.packages("tibble")
library(tibble)

install.packages("dplyr")
library(dbplyr)

install.packages("viridis")
library(viridis)

install.packages("cowplot")
library(cowplot)

install.packages("tidyverse")
library(tidyverse)




# Set work directory to directory of this script
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

# filenames
DFG <- "different_fertilization_rates.pdf"
HFG <- "Hamiltonian_fertilization_rates.pdf"

# ================== #
# Parameter settings #
# ================== #
N <- c(seq(1,12),14,16,18,20)
X <- seq(0.5,0.9,0.05)
P <- c(0.01, 0.02, 0.05, 0.1, 0.2)

# ========================================== #
# PSR frequency at t+1 in a demic population #
# ========================================== #

Pprime <- function(N, P, x = 1 - ((N-1)*(2*N-1)/(N*(4*N-1))))
{
  k <- 1:N
  Pt <- sum((factorial(N)/(factorial(k)*factorial(N-k))) *
    ((P^k) * (1-P)^(N-k)) *
    ((k*x)/(k*x+N*(1-x))) *
    ((N-k)*x)/
    (N*x*(1-P)))
  Pt
}

# ================================================= #
# Spread of PSR under different fertilization rates #
# ================================================= #

d <- matrix(data = NA, nrow = 0, ncol = 4)
for(n in c(1:10,12,15,20,30,50,100))
{
 
  for(x in X)
  {
    p_t <- 0.1
    for(t in 1:200)
    {
      p_t <- Pprime(n,p_t, x)
      d <- rbind(d, cbind(x, t, n, p_t))
    }
  }
}
d <- as_tibble(d)

# PSR dynamics graph
p1 <- d %>% mutate(n = factor(n),
             x = factor(x)) %>%
  ggplot(aes(t, p_t, color = x)) + geom_line(size = 1) + facet_wrap(~n) + 
  scale_x_continuous(breaks = seq(0, 200, 50), expand = c(0.05,0.05)) + 
  scale_y_continuous(limits = c(0,1), breaks = seq(0,1,0.1), expand = c(0.01,0.01)) +
  geom_abline(slope = 0, intercept = 0, linetype = 2) + theme(panel.spacing.x = unit(1, "lines")) +
  scale_color_viridis(begin = 0.3, end = 0.8, option = "A", discrete = T, direction = -1) + 
  labs(title = "PSR dynamics", subtitle = "Variable fertilization rates", x = "Time (generations)", y = "PSR frequency", color = "Fertilization rate (x)")

# PSR equilibrium graph
de <- d %>% filter(t == max(d$t))
p2 <- de %>% mutate(x = factor(x)) %>% 
  ggplot(aes(n, p_t, color = x, group = interaction(n,x))) + 
  geom_point(size = 3) + geom_line(aes(group = x), size = 1) + 
  scale_x_log10(limits = c(1,100), breaks = unique(de$n), expand = c(0.05,0.05)) + 
  scale_y_continuous(limits = c(0, 1), breaks = seq(0.0,1,.1), expand = c(0.01,0.01)) + 
  scale_color_viridis(begin = 0.1, end = 0.8, option = "A", discrete = T, direction = -1) + 
  labs(title = "PSR equilibrium frequencies", subtitle = "Variable fertilization rates", x = "No. foundresses", y = "PSR frequency", color = "Fertilization rate (x)")

legend <- get_legend(p1)

pd <- plot_grid(plotlist = list(p1 + theme(legend.position = "none"), p2 + theme(legend.position = "none"), legend), ncol = 3, rel_widths = c(1,1,0.3))

pdf(file = DFG, height = 12, width = 18)
pd
dev.off()
pd

# ========================================== #
# Spread of PSR under Hamiltonian sex ratios #
# ========================================== #

d <- matrix(data = NA, nrow = 0, ncol = 4)
for(n in N)
{
  for(p in P)
  {
    p_t <- p
    for(t in 1:400)
    {
      p_t <- Pprime(n,p_t)
      d <- rbind(d, cbind(p, t, n, p_t))
    }
  }
}
d <- as_tibble(d)

# PSR dynamics graph
p3 <- d %>% mutate(n = factor(n),
             p = factor(p)) %>%
  ggplot(aes(t, p_t, color = p)) + geom_line(size = 1) + facet_wrap(~n) + 
  scale_x_continuous(breaks = seq(0, 400, 100), expand = c(0.01,0.01)) + 
  scale_y_continuous(breaks = seq(0.0,1,0.1), expand = c(0.01,0.01)) +
  geom_abline(slope = 0, intercept = 0, linetype = 2) + theme(panel.spacing.x = unit(1, "lines")) + 
  scale_color_viridis(begin = 0.3, end = 0.8, option = "A", discrete = T, direction = -1) + 
  labs(title = "PSR dynamics", subtitle = "Hamiltonian ESS fertilization rates", x = "Time (generations)", y = "PSR frequency", color = "Initial PSR frequency")

# PSR equilibrium graph
de <- d %>% filter(t == max(d$t))
p4 <- de %>% mutate(p = factor(p)) %>% 
  ggplot(aes(n, p_t, color = p, group = p)) + 
  geom_line(size = 1) + geom_point(size = 3) + 
  scale_x_log10(limits = c(1,max(N)), breaks = unique(de$n), expand = c(0.01,0.01)) + 
  scale_y_continuous(limits = c(0, 0.05), breaks = seq(0.0,0.05,.01)) + 
  scale_color_viridis(begin = 0.3, end = 0.8, option = "A", discrete = T, direction = -1) +
  labs(title = "PSR equilibrium frequencies", subtitle = "Hamiltonian ESS fertilization rates", x = "No. foundresses", y = "PSR frequency", color = "Initial PSR frequency")

legend <- get_legend(p3)
ph <- plot_grid(plotlist = list(p3 + theme(legend.position = "none"), p4 + theme(legend.position = "none"), legend), ncol = 3, rel_widths = c(1,1,0.3))


pdf(file = HFG, height = 12, width = 18)
ph
dev.off()
ph

